﻿Imports System.Drawing.Drawing2D

Module ChartModule

    Friend Chart1 As Chart = New Chart

    Friend ChartArray1 As ChartArray = New ChartArray

End Module

Public Class ChartArray

    Friend Structure Item
        Dim RegionName As String
        Dim ValueX As Single
        Dim ValueY As Single
        Dim TraceDay As Int32
        Sub New(ByVal _RegionName As String, _
                ByVal _ValueX As Single, _
                ByVal _ValueY As Single, _
                ByVal _TraceDay As Int32)
            RegionName = _RegionName
            ValueX = _ValueX
            ValueY = _ValueY
            TraceDay = _TraceDay
        End Sub
    End Structure

    Friend ChartList As List(Of Item) = New List(Of Item)

    Friend Sub AddItem(ByVal _RegionName As String, _
                       ByVal _ValueX As Single, _
                       ByVal _ValueY As Single, _
                       ByVal _TraceDay As Int32)

        ChartList.Add(New Item(_RegionName, _ValueX, _ValueY, _TraceDay))
    End Sub

    Friend Sub ClearAll()
        ChartList.Clear()
    End Sub

End Class


Public Class Chart

    Friend PB_ChartGraph As PictureBox

    Private BackColor As Color = Color.FromArgb(250, 240, 210) ' 240, 230, 200
    Private Brush_Back As Brush = New SolidBrush(Color.FromArgb(240, 240, 220))

    Private Pen_Scale1 As Pen = New Pen(Color.FromArgb(150, 120, 120))
    Private Pen_Scale1b As Pen = New Pen(Color.FromArgb(150, 160, 160))
    Private Pen_Scale2 As Pen = New Pen(Color.FromArgb(200, 200, 200))
    Private Pen_Scale3 As Pen = New Pen(Color.FromArgb(220, 220, 220))

    Private Brush_Text As Brush = New SolidBrush(Color.FromArgb(50, 50, 150))
    Private Brush_Names As Brush = New SolidBrush(Color.FromArgb(80, 80, 80))
    Private Brush_NamesLight As Brush = New SolidBrush(Color.FromArgb(80, 80, 80, 80))
    Private Brush_Dots As Brush = New SolidBrush(Color.FromArgb(180, 0, 0))
    Private Brush_DotsLight As Brush = New SolidBrush(Color.FromArgb(40, 180, 0, 0))

    Private Font_Text As Font = New Font("Arial", 12, FontStyle.Bold)
    Private Font_Scale As Font = New Font("Arial", 9, FontStyle.Bold)

    Private Font_Date As Font = New Font("Arial", 14, FontStyle.Bold)
    Private Font_Captions As Font = New Font("Arial", 11, FontStyle.Bold)
    Private Brush_Captions As Brush = New SolidBrush(Color.FromArgb(50, 50, 50))

    Friend DayCaption As String

    Friend Ycaption As String
    Friend Ymin As Double = 0
    Friend Ymax As Double = 20
    Friend Ystep As Double = 5
    Friend Ypercent As String = "%"

    Friend Xcaption As String
    Friend Xmin As Double = 0
    Friend Xmax As Double = 10
    Friend Xstep As Double = 5
    Friend Xpercent As String = ""

    Friend Ylog As Boolean
    Friend Xlog As Boolean

    Private hpix As Int32
    Private wpix As Int32

    Private BorderLeft As Int32 = 56
    Private BorderRight As Int32 = 8
    Private BorderUp As Int32 = 8
    Private BorderDown As Int32 = 40


    ' =====================================================================================================
    '  SCALE CONVERSION FUNCTIONS
    ' =====================================================================================================
    Private Function ValueToY(ByVal value As Double) As Single
        If Ylog Then
            Return ValueToLogY(value)
        Else
            Return ValueToLinY(value)
        End If
    End Function
    Private Function ValueToX(ByVal value As Double) As Single
        If Xlog Then
            Return ValueToLogX(value)
        Else
            Return ValueToLinX(value)
        End If
    End Function

    Private Function YToValue(ByVal Y As Single) As Double
        If Ylog Then
            Return LogYToValue(Y)
        Else
            Return LinYToValue(Y)
        End If
    End Function
    Private Function XToValue(ByVal X As Single) As Double
        If Xlog Then
            Return LogXToValue(X)
        Else
            Return LinXToValue(X)
        End If
    End Function

    Private Function ValueToLinY(ByVal value As Double) As Single
        Dim hgraph As Int32 = hpix - BorderUp - BorderDown
        value = (value - Ymin) / (Ymax - Ymin)
        Return CSng(hpix - BorderDown - (value * hgraph))
    End Function
    Private Function LinYToValue(ByVal Y As Double) As Double
        Dim hgraph As Int32 = hpix - BorderUp - BorderDown
        Return (Ymax * (hpix - BorderDown - Y) + Ymin * (Y - BorderUp)) / hgraph
    End Function

    Private Function ValueToLinX(ByVal value As Double) As Single
        Dim wgraph As Int32 = wpix - BorderLeft - BorderRight
        value = (value - Xmin) / (Xmax - Xmin)
        Return CSng(BorderLeft + value * wgraph)
    End Function
    Private Function LinXToValue(ByVal X As Double) As Double
        Dim wgraph As Int32 = wpix - BorderLeft - BorderRight
        Return (Xmax * (X - BorderLeft) + Xmin * (wpix - BorderRight - X)) / wgraph
    End Function

    Private Function ValueToLogY(ByVal value As Double) As Single
        If Ymin < 0.001 Then Ymin = 0.001
        If value <= 0 Then Return -999
        Dim hgraph As Int32 = hpix - BorderUp - BorderDown
        Return CSng(BorderUp + hgraph * (1 - Math.Log(Ymin / value) / Math.Log(Ymin / Ymax)))
    End Function
    Private Function LogYToValue(ByVal Y As Double) As Double
        Dim hgraph As Int32 = hpix - BorderUp - BorderDown
        Return Math.E ^ (((hpix - BorderDown - Y) * Math.Log(Ymax) + (Y - BorderUp) * Math.Log(Ymin)) / hgraph)
    End Function

    Private Function ValueToLogX(ByVal value As Double) As Single
        If Xmin < 0.001 Then Xmin = 0.001
        If value <= 0 Then Return -999
        Dim wgraph As Int32 = wpix - BorderLeft - BorderRight
        Return CSng(BorderLeft + wgraph * (Math.Log(Xmin / value) / Math.Log(Xmin / Xmax)))
    End Function
    Private Function LogXToValue(ByVal X As Double) As Double
        Dim wgraph As Int32 = wpix - BorderLeft - BorderRight
        Return Math.E ^ (((X - BorderLeft) * Math.Log(Xmax) + (wpix - BorderRight - X) * Math.Log(Xmin)) / wgraph)
    End Function

    ' =====================================================================================================
    '  MOUSE MOVE
    ' =====================================================================================================
    Friend SelectedRegion As String = ""
    Friend LockedRegion As Boolean
    Private ArmedRegion As Boolean

    Friend Sub DeselectRegion()
        SelectedRegion = ""
        LockedRegion = False
        ArmedRegion = True
    End Sub

    Friend Sub MouseMove(ByVal e As System.Windows.Forms.MouseEventArgs)
        Dim Region As String = GetItemByXY(e.X, e.Y).RegionName
        If Region <> "" Then
            If ArmedRegion Then
                LockedRegion = False
                ArmedRegion = False
                SelectedRegion = Region
                UpdateChart()
            End If
        Else
            If Not LockedRegion Then
                If SelectedRegion <> "" Then
                    SelectedRegion = ""
                    UpdateChart()
                End If
                ArmedRegion = True
            End If
        End If
    End Sub

    Friend Sub MouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        Dim Region As String = GetItemByXY(e.X, e.Y).RegionName
        If Region = "" Then Return
        If LockedRegion And Region = SelectedRegion Then
            LockedRegion = False
            SelectedRegion = ""
            UpdateChart()
        Else
            LockedRegion = True
            SelectedRegion = Region
            UpdateChart()
        End If
    End Sub

    Friend Function GetItemByXY(ByVal x As Single, ByVal y As Single) As ChartArray.Item
        Dim dX As Single
        Dim dY As Single
        Dim dXmin As Single = 9.0E+99
        Dim dYmin As Single = 9.0E+99
        Dim itm As ChartArray.Item = Nothing
        For Each item As ChartArray.Item In ChartArray1.ChartList
            If item.TraceDay = 0 Then
                dX = Math.Abs(x - ValueToX(item.ValueX))
                dY = Math.Abs(y - ValueToY(item.ValueY))
                If dX < 10 And dY < 10 Then
                    If dX < dXmin And dY < dYmin Then
                        dXmin = dX
                        dYmin = dY
                        itm = item
                    End If
                End If
            End If
        Next
        Return itm
    End Function

    ' =====================================================================================================
    '  UPDATE CHART
    ' =====================================================================================================
    Friend Sub UpdateChart()
        ' ---------------------------------------------------------------------------
        If Not PB_ChartGraph.Visible Then Return
        InitPictureboxImage(PB_ChartGraph)
        If PB_ChartGraph.Image Is Nothing Then Return
        ' ---------------------------------------------------------------------------
        Dim g As Graphics = Graphics.FromImage(PB_ChartGraph.Image)
        g.Clear(BackColor)

        g.SmoothingMode = SmoothingMode.AntiAlias
        g.TextRenderingHint = Drawing.Text.TextRenderingHint.AntiAliasGridFit
        ' ---------------------------------------------------------------------------
        hpix = PB_ChartGraph.Image.Height
        wpix = PB_ChartGraph.Image.Width
        Dim hgraph As Int32 = hpix - BorderUp - BorderDown
        Dim wgraph As Int32 = wpix - BorderLeft - BorderRight
        If hpix < 1 Or wpix < 1 Then Return
        '
        ' --------------------------------------------------------------------------- draw borders
        g.FillRectangle(Brush_Back, BorderLeft, BorderUp, wgraph, hgraph)
        g.DrawRectangle(Pen_Scale1, BorderLeft - 1, BorderUp - 1, wgraph + 2, hgraph + 2)
        '
        ' --------------------------------------------------------------------------- draw scales
        If Xlog Then DrawScaleX_LOG(g) Else DrawScaleX_LIN(g)
        If Ylog Then DrawScaleY_LOG(g) Else DrawScaleY_LIN(g)
        '
        ' --------------------------------------------------------------------------- draw day caption
        g.DrawString(DayCaption, Font_Date, Brush_Captions, _
                                 wpix - BorderRight - 120, BorderUp + 4)
        '
        ' --------------------------------------------------------------------------- draw X-axis caption
        g.DrawString(Xcaption, Font_Captions, Brush_Captions, _
                     BorderLeft + wgraph / 2.0F - 140, hpix - 18)
        ' --------------------------------------------------------------------------- draw Y-axis caption
        g.RotateTransform(-90)
        g.DrawString(Ycaption, Font_Captions, Brush_Captions, -hpix / 2.0F - 120, 2)
        g.ResetTransform()
        '
        ' --------------------------------------------------------------------------- draw chart items
        Dim px, py As Single
        Dim brnames As Brush
        Dim brdots As Brush
        Dim brtrace As Brush
        For Each item As ChartArray.Item In ChartArray1.ChartList
            If item.ValueX > 0.001 And item.ValueY > 0.001 Then
                ' -------------------------------------------------------------------
                Dim n As Int32 = item.TraceDay * 6
                brtrace = New SolidBrush(Color.FromArgb(255, 120 - n, 80 - n))
                brnames = Brush_Names
                brdots = Brush_Dots
                If SelectedRegion <> "" Then
                    If item.RegionName <> SelectedRegion Then
                        brnames = Brush_NamesLight
                        brdots = Brush_DotsLight
                        brtrace = Brushes.LightGray
                    End If
                End If
                ' -------------------------------------------------------------------
                px = ValueToX(item.ValueX)
                py = ValueToY(item.ValueY)
                If item.TraceDay = 0 Then
                    g.FillEllipse(brdots, px - 4, py - 4, 9, 9)
                    g.DrawString(item.RegionName, Font_Text, brnames, px + 7, py - 8)
                Else
                    g.FillEllipse(brtrace, px - 3, py - 3, 7, 7)
                End If
            End If
        Next
        '
        PB_ChartGraph.Invalidate()
    End Sub


    Private Sub DrawScaleX_LIN(ByVal g As Graphics)
        ' --------------------------------------------------------------------------- auto scale
        If Not Form1.chk_LockScales.Checked Then
            Xmax = -Int32.MaxValue
            For Each item As ChartArray.Item In ChartArray1.ChartList
                If item.ValueX > 0.001 And item.ValueY > 0.001 And item.TraceDay = 0 Then
                    If item.ValueX > Xmax Then Xmax = item.ValueX
                End If
            Next
            ' ----------------------------------------------------------------------- some additional space
            Xmax = Xmax * 1.16
            ' ----------------------------------------------------------------------- limit max value
            If Xmax > 105 Then Xmax = 105
        End If
        ' --------------------------------------------------------------------------- calculate the step
        Xstep = 0.01
        If (Xmax - Xmin) / Xstep > 20 Then Xstep = 0.05
        If (Xmax - Xmin) / Xstep > 20 Then Xstep = 0.1
        If (Xmax - Xmin) / Xstep > 20 Then Xstep = 0.5
        If (Xmax - Xmin) / Xstep > 20 Then Xstep = 1
        If (Xmax - Xmin) / Xstep > 20 Then Xstep = 5
        If (Xmax - Xmin) / Xstep > 20 Then Xstep = 10
        If (Xmax - Xmin) / Xstep > 20 Then Xstep = 50
        '
        ' --------------------------------------------------------------------------- draw X axis
        Dim px As Single
        For x As Double = Xmin To Xmax Step Xstep
            px = ValueToX(x)
            g.DrawLine(Pen_Scale2, px, BorderUp, px, hpix - BorderDown)
            g.DrawString(x.ToString(GCI) + Xpercent, Font_Scale, Brush_Text, px - 9, hpix - BorderDown + 2)
        Next
    End Sub

    Private Sub DrawScaleY_LIN(ByVal g As Graphics)
        ' --------------------------------------------------------------------------- auto scale
        If Not Form1.chk_LockScales.Checked Then
            Ymax = -Int32.MaxValue
            For Each item As ChartArray.Item In ChartArray1.ChartList
                If item.ValueX > 0.001 And item.ValueY > 0.001 And item.TraceDay = 0 Then
                    If item.ValueY > Ymax Then Ymax = item.ValueY
                End If
            Next
            ' ----------------------------------------------------------------------- some additional space
            Ymax = Ymax * 1.08
            ' ----------------------------------------------------------------------- limit max value
            If Ymax > 105 Then Ymax = 105
        End If
        ' --------------------------------------------------------------------------- calculate the step
        Ystep = 0.01
        If (Ymax - Ymin) / Ystep > 30 Then Ystep = 0.05
        If (Ymax - Ymin) / Ystep > 30 Then Ystep = 0.1
        If (Ymax - Ymin) / Ystep > 30 Then Ystep = 0.5
        If (Ymax - Ymin) / Ystep > 30 Then Ystep = 1
        If (Ymax - Ymin) / Ystep > 30 Then Ystep = 5
        If (Ymax - Ymin) / Ystep > 30 Then Ystep = 10
        If (Ymax - Ymin) / Ystep > 30 Then Ystep = 50
        '
        ' --------------------------------------------------------------------------- draw Y axis
        Dim py As Single
        For y As Double = Ymin To Ymax Step Ystep
            py = ValueToY(y)
            g.DrawLine(Pen_Scale2, BorderLeft, py, wpix - BorderRight, py)
            g.DrawString(y.ToString(GCI).PadLeft(4) + Ypercent, Font_Scale, Brush_Text, BorderLeft - 40, py - 5)
        Next
    End Sub

    Private Sub DrawScaleX_LOG(ByVal g As Graphics)
        ' --------------------------------------------------------------------------- auto scale
        If Not Form1.chk_LockScales.Checked Then
            Xmax = -Int32.MaxValue
            Xmin = Int32.MaxValue
            For Each item As ChartArray.Item In ChartArray1.ChartList
                If item.ValueX > 0.001 And item.ValueY > 0.001 And item.TraceDay = 0 Then
                    If item.ValueX > Xmax Then Xmax = item.ValueX
                    If item.ValueX < Xmin Then Xmin = item.ValueX
                End If
            Next
            ' ----------------------------------------------------------------------- some additional space
            Xmax = Xmax * 1.16
            Xmin = Xmin * 0.92
            If Xmax / Xmin < 2 Then
                Xmax = Xmax * 2
                Xmin = Xmin * 0.5
            End If
            ' ----------------------------------------------------------------------- limit max value
            If Xmax > 10000 Then Xmax = 10000
        End If
        ' --------------------------------------------------------------------------- draw scale
        Dim x1 As Single
        Dim dec As Single = CSng(10 ^ Math.Round(Math.Log10(Xmin)))
        Dim unit As Single
        While dec <= Xmax
            For unit = 1 To 9 Step 0.5
                Dim v As Single = CSng(Math.Round(1000.0F * dec * unit) / 1000.0F)
                x1 = ValueToLogX(v)
                If x1 >= BorderLeft And x1 <= wpix - BorderRight Then

                    If unit = 1 Or _
                       ((Xmax / Xmin) < 100 And (unit = 2 Or unit = 5)) Or _
                       (Xmax / Xmin) < 5 Then

                        g.DrawLine(Pen_Scale1b, x1, BorderUp, x1, hpix - BorderDown + 1)
                        Dim s As String
                        If v > 999999 Then
                            s = (v / 1000000).ToString("0") + "M"
                        ElseIf v > 999 Then
                            s = (v / 1000).ToString("0") + "k"
                        Else
                            s = v.ToString(GCI)
                        End If
                        s += Xpercent
                        If x1 > BorderRight + 2 Then
                            Dim w As Single = g.MeasureString(s, Font_Scale).Width
                            g.DrawString(s, Font_Scale, Brush_Text, x1 - w / 2.0F, hpix - BorderDown + 2)
                        End If
                    ElseIf unit Mod 5 = 0 Then
                        g.DrawLine(Pen_Scale2, x1, BorderUp, x1, hpix - BorderDown + 1)
                    Else
                        If CInt(unit) = unit Then
                            g.DrawLine(Pen_Scale3, x1, BorderUp, x1, hpix - BorderDown + 1)
                        End If
                    End If
                End If
                If v >= Xmax Then Exit For
            Next
            dec *= 10
        End While
    End Sub

    Private Sub DrawScaleY_LOG(ByVal g As Graphics)
        ' --------------------------------------------------------------------------- auto scale
        If Not Form1.chk_LockScales.Checked Then
            Ymax = -Int32.MaxValue
            Ymin = Int32.MaxValue
            For Each item As ChartArray.Item In ChartArray1.ChartList
                If item.ValueX > 0.001 And item.ValueY > 0.001 And item.TraceDay = 0 Then
                    If item.ValueY > Ymax Then Ymax = item.ValueY
                    If item.ValueY < Ymin Then Ymin = item.ValueY
                End If
            Next
            ' ----------------------------------------------------------------------- some additional space
            Ymax = Ymax * 1.08
            Ymin = Ymin * 0.92
            If Ymax / Ymin < 4 Then
                Ymax = Ymax * 2
                Ymin = Ymin * 0.5
            End If
            ' ----------------------------------------------------------------------- limit max value
            If Ymax > 10000 Then Ymax = 10000
        End If
        ' --------------------------------------------------------------------------- draw scale
        Dim y1 As Single
        Dim dec As Single = CSng(10 ^ Math.Round(Math.Log10(Ymin)))
        Dim unit As Single
        While dec <= Ymax
            For unit = 1 To 9 Step 0.5
                Dim v As Single = CSng(Math.Round(1000.0F * dec * unit) / 1000.0F)
                y1 = ValueToLogY(v)
                If y1 >= BorderUp And y1 <= hpix - BorderDown Then

                    If unit = 1 Or _
                       ((Ymax / Ymin) < 100 And (unit = 2 Or unit = 5)) Or _
                       (Ymax / Ymin) < 5 Then

                        g.DrawLine(Pen_Scale1b, BorderLeft, y1, wpix - BorderRight, y1)
                        Dim s As String
                        If v > 999999 Then
                            s = (v / 1000000).ToString("0") + "M"
                        ElseIf v > 999 Then
                            s = (v / 1000).ToString("0") + "k"
                        Else
                            s = v.ToString(GCI)
                        End If
                        s += Ypercent
                        If y1 > BorderUp + 2 Then
                            Dim w As Single = g.MeasureString(s, Font_Scale).Width
                            g.DrawString(s, Font_Scale, Brush_Text, BorderLeft - 2 - w, y1 - 8)
                        End If
                    ElseIf unit Mod 5 = 0 Then
                        g.DrawLine(Pen_Scale2, BorderLeft, y1, wpix + 1, y1)
                    Else
                        If CInt(unit) = unit Then
                            g.DrawLine(Pen_Scale3, BorderLeft, y1, wpix + 1, y1)
                        End If
                    End If
                End If
                If v >= Ymax Then Exit For
            Next
            dec *= 10
        End While
    End Sub

End Class
