﻿Imports System.Drawing.Drawing2D

Module GraphPaperModule

    Friend TimeGraphRefreshFlag As Boolean = True

    Friend MaxHours As Int32 = 24
    Friend RowTimeMS As Int32 = 100

    Friend ScaleMax As Single = 400000
    Friend ScaleMin As Single = 1
    Friend ScaleHorizontal As Single = 0.2

    Friend DeltaSamples As Single = 30

    Friend DataArray1 As DataArray

    Friend GraphPaper1 As GraphPaper = New GraphPaper

    Friend Sub RedimDataArrayMax(ByVal n As Int32)
        DataArray1 = New DataArray(n)
    End Sub

    Friend Sub LimitDeltaSamples()
        Dim MaxDelta As Single = DataArray1.GetNumEntries - 1
        If DeltaSamples > MaxDelta Then DeltaSamples = MaxDelta
        If DeltaSamples < 0 Then DeltaSamples = 0
    End Sub

    Friend Function MillisecToHourMinAndSec(ByVal millisec As Single) As String
        If millisec >= 3600000 Then
            Dim ts As TimeSpan = TimeSpan.FromMilliseconds(millisec)
            Return Math.Floor(ts.TotalHours).ToString() + "h " + _
                   Math.Floor(ts.Minutes).ToString() + "m " + _
                              ts.Seconds.ToString + "s"
        ElseIf millisec >= 60000 Then
            Dim ts As TimeSpan = TimeSpan.FromMilliseconds(millisec)
            Return Math.Floor(ts.TotalMinutes).ToString() + "m " _
                                    + ts.Seconds.ToString + "s"
        Else
            Return (millisec / 1000).ToString("0", GCI) + " Sec"
        End If
    End Function

End Module

Public Class DataArray

    Friend Structure DataSample
        Dim V1 As Single
        Dim V2 As Single
        Dim V3 As Single
        Dim Time As DateTime
        Sub New(ByVal _v1 As Single, _
                ByVal _v2 As Single, _
                ByVal _v3 As Single, _
                ByVal _time As DateTime)
            V1 = _v1
            V2 = _v2
            V3 = _v3
            Time = _time
        End Sub
    End Structure

    Private m_ArrayMaxLength As Int32
    Private m_WriteIndex As Int32
    Private m_DataArray() As DataSample
    Private m_NumEntries As Int32

    Friend Sub New(ByVal ArrayMaxLength As Int32)
        m_ArrayMaxLength = ArrayMaxLength
        ReDim m_DataArray(m_ArrayMaxLength - 1)
        Flush()
    End Sub

    Friend Sub Flush()
        m_WriteIndex = 0
        m_NumEntries = 0
    End Sub

    Friend Function GetWriteIndex() As Int32
        Return m_WriteIndex
    End Function

    Friend Sub SetWriteIndex(ByVal WriteIndex As Int32)
        If WriteIndex < 0 Then WriteIndex = 0
        If WriteIndex > m_ArrayMaxLength - 1 Then WriteIndex = m_ArrayMaxLength - 1
        m_WriteIndex = WriteIndex
    End Sub

    Friend Sub SetNumEntries(ByVal n As Int32)
        If n < 0 Then n = 0
        If n > m_ArrayMaxLength - 1 Then n = m_ArrayMaxLength - 1
        m_NumEntries = n
    End Sub

    Friend Function GetNumEntries() As Int32
        Return m_NumEntries
    End Function

    Friend Function GetBuffer() As DataSample()
        Return m_DataArray
    End Function

    Friend Sub AddElement(ByVal V1 As Double, _
                          ByVal V2 As Double, _
                          ByVal V3 As Double, _
                          ByVal Time As DateTime)
        Dim ds As DataSample
        ds = New DataSample(CSng(V1), CSng(V2), CSng(V3), Time)
        m_DataArray(m_WriteIndex) = ds
        m_WriteIndex += 1
        If m_WriteIndex > m_ArrayMaxLength - 1 Then m_WriteIndex = m_ArrayMaxLength - 1
        m_NumEntries += 1
        If m_NumEntries > m_ArrayMaxLength Then m_NumEntries = m_ArrayMaxLength
    End Sub

    Friend Sub FillWithInvalidData()
        Dim ds As DataSample = New DataSample(-999, -999, -999, DateTime.MinValue)
        For i As Int32 = 0 To m_ArrayMaxLength - 1
            m_DataArray(i) = ds
        Next
    End Sub

    Friend Function GetElement(ByVal Index As Int32) As DataSample
        If Index < 0 Then Index = 0
        If Index > m_ArrayMaxLength - 1 Then Index = m_ArrayMaxLength - 1
        Return m_DataArray(Index)
    End Function

End Class


Public Class GraphPaper

    Friend PB_TimeGraph As PictureBox
    Friend LanguageIsEnglish As Boolean = True

    Private BackColor As Color = Color.FromArgb(250, 240, 210) ' 240, 230, 200
    Private Pen_Scale1 As Pen = New Pen(Color.FromArgb(150, 120, 120))
    Private Pen_Scale2 As Pen = New Pen(Color.FromArgb(200, 200, 200))
    Private Pen_Scale3 As Pen = New Pen(Color.FromArgb(220, 220, 220))

    Private Pen_CenterDivision As Pen = New Pen(Color.FromArgb(255, 255, 255, 100), 11)
    Private Pen_Hours As Pen = New Pen(Color.FromArgb(180, 100, 100), 5)
    Private Pen_Minutes As Pen = New Pen(Color.FromArgb(120, 180, 120), 3)
    Private Pen_Seconds As Pen = New Pen(Color.FromArgb(150, 150, 220))
    Private Pen_TimeGap As Pen = New Pen(Color.FromArgb(150, 150, 150), 20)

    Private Pen_Trace1 As Pen = New Pen(Color.FromArgb(60, 60, 180), 4)
    Private Pen_Trace2 As Pen = New Pen(Color.FromArgb(0, 150, 0), 4)
    Private Pen_Trace3 As Pen = New Pen(Color.FromArgb(150, 0, 0), 4)

    Private Brush_Trace1 As Brush = New SolidBrush(Color.FromArgb(40, 40, 180))
    Private Brush_Trace2 As Brush = New SolidBrush(Color.FromArgb(0, 120, 0))
    Private Brush_Trace3 As Brush = New SolidBrush(Color.FromArgb(150, 40, 40))

    Private Brush_Text As Brush = New SolidBrush(Color.FromArgb(50, 50, 50))
    Private Brush_HilightBack As Brush = New SolidBrush(Color.FromArgb(150, 255, 255, 120))
    Private Brush_HilightText As Brush = New SolidBrush(Color.FromArgb(60, 60, 60))

    Private Font_Text As Font = New Font("Arial", 10, FontStyle.Bold)
    Private Font_Scale As Font = New Font("Arial", 9, FontStyle.Bold)
    Private Font_Labels As Font = New Font("Arial", 10, FontStyle.Bold)

    ' ==============================================================================
    '  UpdateDeltaSamples by DeltaPixels
    ' ==============================================================================
    Friend Sub UpdateWithMousePressed(ByVal DeltaX As Int32, ByVal DeltaY As Int32, ByVal mouseX As Int32, ByVal mouseY As Int32)
        If mouseX < BorderLeft Then
            If mouseY < PB_TimeGraph.ClientSize.Height * 0.5 Then
                ScaleMax *= (1.0F + DeltaY * 0.03F)
                If ScaleMax < ScaleMin * 4 Then ScaleMax = ScaleMin * 4
                If ScaleMax > 200000000 Then ScaleMax = 200000000
            Else
                ScaleMin *= (1.0F + DeltaY * 0.03F)
                If ScaleMin > 100 Then ScaleMin = 100
                If ScaleMin < 0.01 Then ScaleMin = 0.01
                ' ------------------------------------------------------- lock to 0.1 / 1 / 10
                If Math.Abs(ScaleMin - 10) < 0.5 Then ScaleMin = 10
                If Math.Abs(ScaleMin - 1) < 0.05 Then ScaleMin = 1
                If Math.Abs(ScaleMin - 0.1) < 0.005 Then ScaleMin = 0.1
                ' ------------------------------------------------------- limit
                If ScaleMax < ScaleMin * 4 Then ScaleMax = ScaleMin * 4
            End If
        ElseIf mouseY > hpix - BorderDown Then
            Dim xs As Single = ScaleHorizontal
            xs = xs * (1.0F - DeltaX * 0.01F)
            If xs < 0.02 Then xs = 0.02
            If xs > 2000 Then xs = 2000
            ScaleHorizontal = xs
        Else
            DeltaSamples += DeltaX * ScaleHorizontal
            LimitDeltaSamples()
        End If
    End Sub

    Friend Sub UpdateScalesWithMouseWheel(ByVal mouseX As Int32, ByVal mouseY As Int32, ByVal delta As Int32)
        If mouseX < BorderLeft Then
            ScaleMax *= (1.0F - delta * 0.5F)
            If ScaleMax < ScaleMin * 6 Then ScaleMax = ScaleMin * 6
            If ScaleMax > 200000000 Then ScaleMax = 200000000
        Else
            Dim xs As Single = ScaleHorizontal
            xs = xs * (1.0F - delta * 0.3F)
            If xs < 0.02 Then xs = 0.02
            If xs > 2000 Then xs = 2000
            ScaleHorizontal = xs
            LimitDeltaSamples()
        End If
    End Sub

    Friend Sub ShowTimePoint(ByVal time As DateTime)
        For i As Int32 = 0 To DataArray1.GetNumEntries - 1
            If DataArray1.GetElement(i).Time.Subtract(time).TotalMilliseconds >= 0 Then
                'Form1.Text = i.ToString
                DeltaSamples = DataArray1.GetNumEntries - i - 1
                TimeGraphRefreshFlag = True
                Exit For
            End If
        Next
    End Sub


    ' ==============================================================================
    '  TIME GRAPH
    ' ==============================================================================
    Private hpix As Int32
    Private wpix As Int32

    Private BorderLeft As Int32 = 36
    Private BorderUp As Int32 = 4
    Private BorderDown As Int32 = 20

    Friend Function ValueToLogY(ByVal value As Double) As Single
        If value <= 0 Then Return -999
        Dim hgraph As Int32 = hpix - BorderUp - BorderDown
        Return CSng(BorderUp + hgraph * (1 - Math.Log(ScaleMin / value) / Math.Log(ScaleMin / ScaleMax)))
    End Function
    Public Function LogYToValue(ByVal Y As Double) As Double
        Dim h As Double = hpix - BorderDown - BorderUp
        Return ScaleMin ^ ((BorderUp - Y) / h) * ScaleMax ^ ((hpix - BorderDown - Y) / h)
    End Function

    Friend Sub UpdateTimeGraph()
        '
        'Dim sw1 As Diagnostics.Stopwatch = New Diagnostics.Stopwatch : sw1.Start()
        ' --------------------------------------------------------------------- 
        If Not PB_TimeGraph.Visible Then Return
        InitPictureboxImage(PB_TimeGraph)
        If PB_TimeGraph.Image Is Nothing Then Return
        ' --------------------------------------------------------------------- 
        Dim v1, v2, v3 As Single
        Dim y1, y2, y3 As Single
        Dim oldx As Int32
        Dim oldy1, oldy2, oldy3 As Single
        Dim oldv1, oldv2, oldv3 As Single
        Dim oldTime As DateTime = DateTime.MinValue
        Dim strTime As String = ""
        Dim strInfo1 As String = ""
        Dim strInfo2 As String = ""
        Dim strInfo3 As String = ""
        Dim numTraces As Int32 = 2
        Dim xmid As Single
        'Dim MaxTimeGap As Single = 2 + ScaleHorizzontal * RowTimeMS / 500.0F
        ' ---------------------------------------------------------------------
        hpix = PB_TimeGraph.Image.Height
        wpix = PB_TimeGraph.Image.Width
        xmid = CInt(BorderLeft + (wpix - BorderLeft) / 2.0F)
        ' ---------------------------------------------------------------------
        If hpix < 1 Or wpix < 1 Then Return
        ' ---------------------------------------------------------------------
        Dim g As Graphics = Graphics.FromImage(PB_TimeGraph.Image)
        g.Clear(BackColor)
        If ScaleMin >= ScaleMax Then
            g.DrawString(" INVALID PARAMS", Font_Scale, Brushes.Black, 24, 24)
            Return
        End If
        ' --------------------------------------------------------------------- file not found
        If DataArray1 Is Nothing Then
            g.DrawString(" FILE NOT FOUND", Font_Scale, Brushes.Black, 24, 24)
            PB_TimeGraph.Refresh()
            Return
        End If
        ' --------------------------------------------------------------------- array read index with DeltaSamples
        Dim ArrayReadIndexCentral As Int32
        Dim ArrayReadIndexLeft As Single
        Dim ArrayReadIndexInt As Int32
        Static oldArrayReadIndexInt As Int32
        Dim CentralSample As DataArray.DataSample = Nothing
        Dim ClearanceLeft As Int32 = 2
        '
        ' --------------------------------------------------------------------- ArrayReadIndexCentral and CentralSample
        ArrayReadIndexCentral = CInt(DataArray1.GetNumEntries - 1 - DeltaSamples)
        CentralSample = DataArray1.GetElement(ArrayReadIndexCentral)
        '
        ' --------------------------------------------------------------------- ArrayReadIndexLeft
        ArrayReadIndexLeft = ArrayReadIndexCentral - ScaleHorizontal * (xmid - BorderLeft - ClearanceLeft)
        '
        CentralSample = DataArray1.GetElement(ArrayReadIndexCentral)
        '
        ' --------------------------------------------------------------------- stabilize and suppress aliasing effect
        ArrayReadIndexLeft = CInt(ArrayReadIndexLeft / ScaleHorizontal) * ScaleHorizontal
        '
        ' --------------------------------------------------------------------- draw center division
        g.DrawLine(Pen_CenterDivision, xmid - 1, 18, xmid - 1, hpix - BorderDown)
        '
        ' --------------------------------------------------------------------- draw scale y
        'g.DrawString("PPM", Font_Scale, Brushes.Black, 1, 1)
        Dim dec As Single = CSng(10 ^ Math.Round(Math.Log10(ScaleMin)))
        Dim unit As Single
        While dec <= ScaleMax
            For unit = 1 To 9 Step 0.5
                Dim v As Single = CSng(Math.Round(1000.0F * dec * unit) / 1000.0F)
                y1 = ValueToLogY(v)
                If y1 <= hpix - BorderDown Then
                    If unit = 1 Or ((ScaleMax / ScaleMin) < hpix * 5 And (unit = 2 Or unit = 5)) Or _
                                    (ScaleMax / ScaleMin) < hpix * 0.02 Then
                        g.DrawLine(Pen_Scale1, BorderLeft, y1, wpix + 1, y1)
                        Dim s As String
                        If v > 999999 Then
                            s = (v / 1000000).ToString("0") + "M"
                        ElseIf v > 999 Then
                            s = (v / 1000).ToString("0") + "k"
                        Else
                            s = v.ToString(GCI)
                        End If
                        If y1 > BorderUp + 2 Then
                            Dim w As Single = g.MeasureString(s, Font_Scale).Width
                            g.DrawString(s, Font_Scale, Brushes.Black, BorderLeft - 2 - w, y1 - 8)
                        End If
                    ElseIf unit Mod 5 = 0 Then
                        g.DrawLine(Pen_Scale2, BorderLeft, y1, wpix + 1, y1)
                    Else
                        If CInt(unit) = unit Then
                            g.DrawLine(Pen_Scale3, BorderLeft, y1, wpix + 1, y1)
                        End If
                    End If
                End If
                If v >= ScaleMax Then Exit For
            Next
            dec *= 10
        End While
        ' --------------------------------------------------------------------- draw borders
        g.DrawLine(Pen_Scale1, BorderLeft + 2, BorderUp, BorderLeft + 2, hpix - BorderDown)
        g.DrawLine(Pen_Scale1, BorderLeft, hpix - BorderDown, wpix + 1, hpix - BorderDown)
        ' --------------------------------------------------------------------- draw all the waves
        Dim MidPoint As Int32 = CInt(wpix / 2.0) - 1
        g.SmoothingMode = Drawing2D.SmoothingMode.HighQuality
        oldx = -1
        oldy1 = 0
        oldy2 = 0
        oldy3 = 0
        oldv1 = 0
        oldv2 = 0
        oldv3 = 0
        For ix As Int32 = BorderLeft + ClearanceLeft To wpix + CInt(1 / ScaleHorizontal)
            ' ----------------------------------------------------------------- 
            ArrayReadIndexLeft += ScaleHorizontal
            ArrayReadIndexInt = CInt(ArrayReadIndexLeft - 0.5)
            If ArrayReadIndexInt < 0 Then Continue For
            If ArrayReadIndexInt > DataArray1.GetNumEntries - 1 Then
                ArrayReadIndexInt = DataArray1.GetNumEntries - 1
            End If
            ' ----------------------------------------------------------------- 
            With DataArray1.GetElement(ArrayReadIndexInt)
                v1 = .V1
                v2 = .V2
                v3 = .V3
                If v3 > 0 Then numTraces = 3
                If v1 < ScaleMin Then v1 = ScaleMin
                If v2 < ScaleMin Then v2 = ScaleMin
                If v3 < ScaleMin Then v3 = ScaleMin
                ' ------------------------------------------------------------- draw Hour/Min/Sec divisions
                Dim yup As Single = hpix - BorderDown - 10
                Dim ytxt As Single = hpix - BorderDown + 4
                If oldTime > DateTime.MinValue Then
                    If ScaleHorizontal > 1 Then
                        If .Time.Year <> oldTime.Year Then
                            g.DrawLine(Pen_Hours, ix, yup, ix, hpix - BorderDown)
                            g.DrawString(.Time.ToString("YYY"), Font_Scale, Brush_Text, ix - 9, ytxt)
                        End If
                    ElseIf ScaleHorizontal > 0.05 Then
                        If .Time.Month <> oldTime.Month Then
                            g.DrawLine(Pen_Minutes, ix, yup, ix, hpix - BorderDown)
                            g.DrawString(.Time.ToString("MMM"), Font_Scale, Brush_Text, ix - 7, ytxt)
                        End If
                        If ScaleHorizontal < 0.15 Then
                            If .Time.Day <> 1 AndAlso .Time.Day \ 5 <> oldTime.Day \ 5 Then
                                g.DrawLine(Pen_Seconds, ix, yup, ix, hpix - BorderDown)
                                g.DrawString(.Time.Day.ToString.PadLeft(2), Font_Scale, Brush_Text, ix - 8, ytxt)
                            End If
                        ElseIf ScaleHorizontal < 0.3 Then
                            If .Time.Day <> 1 And .Time.Day <> 30 AndAlso .Time.Day \ 10 <> oldTime.Day \ 10 Then
                                g.DrawLine(Pen_Seconds, ix, yup, ix, hpix - BorderDown)
                                g.DrawString(.Time.Day.ToString.PadLeft(2), Font_Scale, Brush_Text, ix - 8, ytxt)
                            End If
                        End If
                    Else
                        If .Time.Day <> oldTime.Day Then
                            g.DrawLine(Pen_Seconds, ix, yup, ix, hpix - BorderDown)
                            g.DrawString(.Time.Day.ToString.PadLeft(2), Font_Scale, Brush_Text, ix - 8, ytxt)
                        End If
                    End If
                End If

                oldTime = .Time
            End With
            ' ----------------------------------------------------------------- connect samples with diagonal line
            If ArrayReadIndexInt = oldArrayReadIndexInt Then Continue For
            oldArrayReadIndexInt = ArrayReadIndexInt
            ' ----------------------------------------------------------------- exclude blank area
            If v1 = -999 Or v2 = -999 Or v3 = -999 Then Continue For
            ' ----------------------------------------------------------------- y positions
            y1 = ValueToLogY(v1)
            y2 = ValueToLogY(v2)
            y3 = ValueToLogY(v3)
            ' ----------------------------------------------------------------- draw 
            If oldx >= 0 Then
                If oldy1 < 9999 And (v1 > ScaleMin Or oldv1 > ScaleMin) Then g.DrawLine(Pen_Trace1, oldx, oldy1, ix, y1)
                If oldy2 < 9999 And (v2 > ScaleMin Or oldv2 > ScaleMin) Then g.DrawLine(Pen_Trace2, oldx, oldy2, ix, y2)
                If oldy3 < 9999 And (v3 > ScaleMin Or oldv3 > ScaleMin) Then g.DrawLine(Pen_Trace3, oldx, oldy3, ix, y3)
            End If
            ' ----------------------------------------------------------------- set old values
            oldx = ix
            oldy1 = y1
            oldy2 = y2
            oldy3 = y3
            oldv1 = v1
            oldv2 = v2
            oldv3 = v3
        Next
        ' --------------------------------------------------------------------- english or italian
        If LanguageIsEnglish Then
            strTime = CentralSample.Time.ToString("dd MMMM yyyy", Globalization.CultureInfo.CreateSpecificCulture("en-US"))
            strInfo1 = Form1.SelectedDataType + " "
            strInfo2 = "Medium increment "
            strInfo3 = "Deaths over total "
        Else
            strTime = CentralSample.Time.ToString("dd MMMM yyyy", Globalization.CultureInfo.CreateSpecificCulture("it-IT"))
            strInfo1 = Form1.SelectedDataType + " "
            strInfo2 = "Incremento medio "
            strInfo3 = "Decessi su totale "
        End If
        ' --------------------------------------------------------------------- draw central time text
        DrawHilightedString(g, strTime, Brush_HilightText, _
                            xmid, 0, HorizontalAlignment.Center)
        ' --------------------------------------------------------------------- draw values
        DrawHilightedString(g, strInfo1 + CentralSample.V1.ToString("0", GCI), _
                            Brush_Trace1, BorderLeft + 8, 4)
        DrawHilightedString(g, strInfo2 + CentralSample.V2.ToString("0", GCI), _
                            Brush_Trace2, wpix + 2, 4, HorizontalAlignment.Right)
        If numTraces = 3 Then
            DrawHilightedString(g, strInfo3 + CentralSample.V3.ToString("0.0", GCI) + "%", _
                                Brush_Trace3, wpix + 6, 22, HorizontalAlignment.Right)
        End If
        ' --------------------------------------------------------------------- show
        PB_TimeGraph.Invalidate()
        'ReportString = "Scope time: " & (sw1.Elapsed.TotalMilliseconds * 1000).ToString("0") & " uS"
        'Form1.Text = "Scope time: " & (sw1.Elapsed.TotalMilliseconds * 1000).ToString("0") & " uS"
    End Sub

    Private Sub DrawHilightedString(ByVal g As Graphics, _
                                    ByVal str As String, _
                                    ByVal br As Brush, _
                                    ByVal x As Single, _
                                    ByVal y As Single, _
                                    Optional ByVal align As HorizontalAlignment = HorizontalAlignment.Left)
        Dim sz As SizeF = g.MeasureString(str, Font_Labels, 999)
        Select Case align
            Case HorizontalAlignment.Left
            Case HorizontalAlignment.Center : x -= sz.Width / 2.0F
            Case HorizontalAlignment.Right : x -= sz.Width
        End Select
        g.FillRectangle(Brush_HilightBack, x - 6, y - 2, sz.Width + 6, sz.Height + 4)
        g.DrawString(str, Font_Labels, br, x, y)
    End Sub

End Class

